<?php
//
//	Module name      :	odbccfg.php
//	Version          :	1.0
//	Started on       :	Maj 4, 2005
//	Completed on     :	Maj 13, 2005
//
//	Module type      :	Library
//	Description      :	Functions to work with a configuration table, shaped as Windows Ini files, in a DB
//						accessed via Odbc module
//
//	Developed under  :	Php 4.1.2
//	Sw requirements  :	None
//
//	Parameters       :	None
//	Notes - usage    :	This module works on a database already initialized via Odbc layer and on a table
//						with definition:
//							CREATE TABLE Configuration (
//								Section varchar(32) NOT NULL default '',
//								Variable varchar(32) NOT NULL default '',
//								Value varchar(255) NOT NULL default '',
//								PRIMARY KEY  (Section,Variable)
//							);
//						Table name and field names can be changed using configuration descriptor in OCInit;
//						field lengths can be different.
//						Section value can be omitted; conventionally, sections with value "" can be considered
//						"general".
//						This module does not have error functions; error codes can be read calling Odbc
//						module functions.
//


//--- Include files --------------------------------------------------------------------------------



require_once("odbc.php");



//--- Public constants -----------------------------------------------------------------------------



define("OC_MODULE_NAME", "odbccfg.php 1.0; on Maj 13, 2005 by J.Hubb.");

// Returned array type
define("OC_STD", FALSE);	// Standard array
define("OC_ASS", TRUE);		// Associative array



//--- Public variables -----------------------------------------------------------------------------



// Example for an uninitialized descriptor
$OC_desc = array(
		"CfgTableName"	=> "MyConfigTable",
		"SecFieldName"	=> "Section",
		"VarFieldName"	=> "Variable",
		"ValFieldName"	=> "Value"
		);



//--- Private constants ----------------------------------------------------------------------------



//--- Private variables ----------------------------------------------------------------------------



$_OC_def_desc = array(
		"CfgTableName"	=> "Configuration",
		"SecFieldName"	=> "Section",
		"VarFieldName"	=> "Variable",
		"ValFieldName"	=> "Value"
		);



//--- Public functions code ------------------------------------------------------------------------



function OCDeleteSection(&$cfg_desc, $sec)
// Deletes, if exist, all the variables in the section $sec; returns FALSE on DB errors, otherwise TRUE.
{
	$val = ODBCQuery($cfg_desc["_DbDesc"],
			"DELETE FROM $cfg_desc[CfgTableName] WHERE $cfg_desc[SecFieldName] = '" . addslashes($sec) . "'");
	if (ODBCResult() !== ODBC_NO_ERROR)
		return FALSE;
	return TRUE;
}



function OCDeleteVar(&$cfg_desc, $sec, $var)
// Deletes, if exists, the variable with name $var, in the section $sec; returns FALSE on DB errors,
// otherwise TRUE.
{
	$val = ODBCQuery($cfg_desc["_DbDesc"],
			"DELETE FROM $cfg_desc[CfgTableName] WHERE $cfg_desc[SecFieldName] = '" . addslashes($sec) .
			"' AND $cfg_desc[VarFieldName] = '" . addslashes($var) . "'");
	if (ODBCResult() !== ODBC_NO_ERROR)
		return FALSE;
	return TRUE;
}



function OCDone(&$desc)
// Closes the reference to specified database. Return nothing.
{
	unset($desc["_DbDesc"]);
	return;
}



function OCInit(&$desc, &$db_desc)
// Initializes the descriptor $desc refererring to the database descriptor $db_desc and setting default
// values if not already set in $desc. Returns nothing.
{
	global $_OC_def_desc;

	$desc = array_merge($_OC_def_desc, $desc);		// Overwrite default values with new ones
	$desc["_DbDesc"] = &$db_desc;
	return;
}



function OCReadSection(&$cfg_desc, $sec, $ass = OC_ASS)
// Reads all the variables in the section $sec and returns them as an array; if $ass is OC_STD the array
// returned is defined as:
//   array(0 => array(0 => "Var1", 1 => Value1),
//         1 => array(0 => "Var2", 1 => Value2),
//         ...);
// if $ass is OC_ASS (default), the array returned is defined as:
//   array("Var1" => Value1,
//         "Var2" => Value2,
//         ...).
// Returns FALSE on DB errors.
{
	$tmp = ODBCSelectManyRows($cfg_desc["_DbDesc"],
			"SELECT $cfg_desc[VarFieldName], $cfg_desc[ValFieldName] FROM $cfg_desc[CfgTableName] WHERE " .
			"$cfg_desc[SecFieldName] = '" . addslashes($sec) . "'");
	if (ODBCResult() !== ODBC_NO_ERROR)
		return FALSE;
	$result = array();
	foreach ($tmp as $row)
		if ($ass)
			$result[$row[0]] = $row[1];
		else
			$result[] = $row;
	return $result;
}



function OCReadSections(&$cfg_desc, $ass = TRUE)
// Reads all the variables in all the sections and returns them as an array; if $ass is OC_STD the array
// returned is defined as:
//   array(0 => array(0 => "Var1", 1 => Value1, 2 => "SectionX"),
//         1 => array(0 => "Var2", 1 => Value2, 2 => "SectionX"),
//         ...);
// if $ass is OC_ASS (default), the array returned is defined as:
//   array("Section1" => array("Var1" => Value1,
//                             "Var2" => Value2,
//                             ...),
//         "Section2" => array(...),
//         ...).
// Returns FALSE on DB errors.
{
	$tmp = ODBCSelectManyRows($cfg_desc["_DbDesc"],
			"SELECT $cfg_desc[VarFieldName], $cfg_desc[ValFieldName], $cfg_desc[SecFieldName] FROM " .
			"$cfg_desc[CfgTableName]");
	if (ODBCResult() !== ODBC_NO_ERROR)
		return FALSE;
	if ($ass)
	{
		$result = array();
		foreach ($tmp as $row)
			$result[$row[2]][$row[0]] = $row[1];
		return $result;
	}
	else
		return $tmp;
}



function OCReadVar(&$cfg_desc, $sec, $var, $default = FALSE)
// Reads the variable with name $var, in the section $sec; if the variable does not exist, it is returned
// the default value $default. Returns FALSE on DB errors.
{
	$val = ODBCSelectOneRow($cfg_desc["_DbDesc"],
			"SELECT $cfg_desc[ValFieldName] FROM $cfg_desc[CfgTableName] WHERE $cfg_desc[SecFieldName] = '" .
			addslashes($sec) . "' AND $cfg_desc[VarFieldName] = '" . addslashes($var) . "'");
	if (ODBCResult() !== ODBC_NO_ERROR)
		return FALSE;
	return (ODBCAffectedRows($cfg_desc["_DbDesc"]) == 1 ? $val[0] : $default);
}



function OCSectionExists(&$cfg_desc, $sec)
// Checks if exists at least one variable in the section $sec. Returns FALSE on DB errors.
{
	list ($val) = ODBCSelectOneRow($cfg_desc["_DbDesc"],
			"SELECT COUNT(*) FROM $cfg_desc[CfgTableName] WHERE $cfg_desc[SecFieldName] = '" .
			addslashes($sec) . "'");
	if (ODBCResult() !== ODBC_NO_ERROR)
		return FALSE;
	return ($val > 0);
}



function OCVarExists(&$cfg_desc, $sec, $var)
// Checks if the variable with name $var, in the section $sec exists. Returns FALSE on DB errors.
{
	$val = OCReadVar($cfg_desc, $sec, $var);
	return ($val !== FALSE ? TRUE : FALSE);
}



function OCWriteVar(&$cfg_desc, $sec, $var, $val)
// Writes the value $val in the variable with name $var, in the section $sec; if the variable does not exist,
// it is created. Returns FALSE on DB errors, otherwise TRUE.
{
	$exists = OCVarExists($cfg_desc, $sec, $var);
	if (ODBCResult() !== ODBC_NO_ERROR)
		return FALSE;
	if ($exists)
	{
		ODBCQuery($cfg_desc["_DbDesc"], "UPDATE $cfg_desc[CfgTableName] SET $cfg_desc[ValFieldName] = '" .
				addslashes($val) . "' WHERE $cfg_desc[SecFieldName] = '" . addslashes($sec) .
				"' AND $cfg_desc[VarFieldName] = '" . addslashes($var) . "'");
		if (ODBCResult() !== ODBC_NO_ERROR)
			return FALSE;
	}
	else
	{
		ODBCQuery($cfg_desc["_DbDesc"],
				"INSERT INTO $cfg_desc[CfgTableName] ($cfg_desc[SecFieldName], $cfg_desc[VarFieldName], " .
				"$cfg_desc[ValFieldName]) VALUES ('" . addslashes($sec) . "', '" . addslashes($var) . "', '" .
				addslashes($val) . "')");
		if (ODBCResult() !== ODBC_NO_ERROR)
			return FALSE;
	}
	return TRUE;
}



//--- Private functions code -----------------------------------------------------------------------



//--- Initialization code --------------------------------------------------------------------------



?>
